//############################################################################
//Addieren von großen Integerzahlen (Blatt03, Aufgabe 1)
//@author diehl@ins.uni-bonn.de
//@date 10-01-2013
//###########################################################################

#include <stdio.h>
#include <stdlib.h>

/**@struct Element einer Liste
 * @var value Wert im Element in der Liste
 * @var next Pointer zum nächsten Element in der Liste
 */
struct element
{
  int value;
  struct element *next;
};

/**Methode gibt die Elemente der Liste aus
 * @param l Pointer auf das erste Element der Liste
 */
void
printList (struct element *l)
{

  struct element *tmp = l;

  while (tmp != NULL)
    {
      printf ("%d ", tmp->value);
      tmp = tmp->next;
    }
  printf ("\n");
}

/** Methode prüft, ob die Liste leer ist.
 * @param l Zeiger auf die Liste
 * @return Liste leer?
 */
int
isEmpty (struct element *l)
{
  int empty = 0;
  if (l == NULL)
    empty = 1;
  return empty;
}

/** Methode fügt das Element value an das Ende der Liste an.
 * @param l Zeiger auf die Liste
 * @param value Wert, der eingefügt werden soll
 */
void
append (struct element **l, int value)
{

  struct element *tmp = *l;

  struct element *new = (struct element *) malloc (sizeof (struct element));
  new->value = value;
  new->next = NULL;


  if (tmp == NULL)
    *l = new;

  else
    {

      while (tmp->next != NULL)
	{
	  tmp = tmp->next;
	}
      tmp->next = new;
    }

}

/**Methode löscht alle Elemente der Liste
 * @param l Pointer zum ersten Element der Listen
 */
void
delete (struct element *l)
{

  struct element *tmp = l->next;

  while (l->next != NULL)
    {
      tmp = l->next;
      l->next = tmp->next;
      free (tmp);
    }
  free (l);

}

/**Methode gibt die Länge der Liste zurück
 * @param l Pointer zum ersten Element der Liste
 * @return Länge der Liste
 */
int
length (struct element *l)
{

  struct element *tmp = l;
  int n = 0;

  while (tmp != NULL)
    {
      ++n;
      tmp = tmp->next;
    }

  return n;
}

/** Methode addiert die Zahlen in der liste zahl1 und der Liste zahl2 und speichert
 * das Ergebnis der Addition in die Liste result.
 * @pre Die beiden Listen sind immer genau gleich lang und werden mit führenden Nullen
 * 	aufgefüllt.
 * @param zahl1 Pointer zur Liste mit der ersten Zahl
 * @param zahl2 Pointer zur Liste mit der zweiten Zahl
 * @param result Pointer zur Liste mit dem Ergebnis der Addition
 */
void
add (struct element *zahl1, struct element *zahl2, struct element **result)
{

  if (isEmpty (zahl1) == 0 && isEmpty (zahl2) == 0)
    {

      struct element *tmp1 = zahl1;
      struct element *tmp2 = zahl2;

      int value = (tmp1->value + tmp2->value) % 10;
      int overflow = (tmp1->value + tmp2->value) / 10;

      append (result, value);

      while (tmp1->next != NULL)
	{
	  tmp1 = tmp1->next;
	  tmp2 = tmp2->next;

	  value = (tmp1->value + tmp2->value + overflow) % 10;
	  overflow = (tmp1->value + tmp2->value + overflow) / 10;
	  append (result, value);

	}

      if (overflow > 0)
	append (result, overflow);

    }

}

int
main (void)
{

  struct element *zahl1 = NULL;
  struct element *zahl2 = NULL;
  struct element *result = NULL;

  printf ("Liste 1 ist leer: %d \n", isEmpty (zahl1));
  printf ("Liste 2 ist leer: %d \n", isEmpty (zahl2));

  // UINT_MAX + UINT_MAX
  append (&zahl1, 5);
  append (&zahl1, 3);
  append (&zahl1, 5);
  append (&zahl1, 5);
  append (&zahl1, 6);

  append (&zahl2, 5);
  append (&zahl2, 3);
  append (&zahl2, 5);
  append (&zahl2, 5);
  append (&zahl2, 6);

  //Addiere die beiden Zahlen
  add (zahl1, zahl2, &result);

  //Prüfe, ob die beiden Listen nicht leer sind
  printf ("Liste 1 ist leer: %d \n", isEmpty (zahl1));
  printf ("Liste 2 ist leer: %d \n", isEmpty (zahl2));

  //Gebe die Länge der Listen aus
  printf ("Liste 1 hat %d Elemente \n", length (zahl1));
  printf ("Liste 2 hat %d Elemente \n", length (zahl2));
  printf ("Ergebnis hat %d Elemente \n", length (result));

  printList (zahl1);
  printList (zahl2);
  printList (result);

  delete (zahl1);
  delete (zahl2);
  delete (result);

  //TODO: Fügen Sie die beiden Zahlen in jeweils eine Liste ein.

  // ULONG_MAX (18446744073709551615) + LLONG_MAX (9223372036854775807)

  zahl1 = NULL;
  zahl2 = NULL;
  result = NULL;
  
  append (&zahl1, 5);
  append (&zahl1, 1);
  append (&zahl1, 6);
  append (&zahl1, 1);
  append (&zahl1, 5);
  append (&zahl1, 5);
  append (&zahl1, 9);
  append (&zahl1, 0);
  append (&zahl1, 7);
  append (&zahl1, 3);
  append (&zahl1, 7);
  append (&zahl1, 0);
  append (&zahl1, 4);
  append (&zahl1, 4);
  append (&zahl1, 7);
  append (&zahl1, 6);
  append (&zahl1, 4);
  append (&zahl1, 4);
  append (&zahl1, 8);
  append (&zahl1, 1);

  append (&zahl2, 7);
  append (&zahl2, 0);
  append (&zahl2, 8);
  append (&zahl2, 5);
  append (&zahl2, 7);
  append (&zahl2, 7);
  append (&zahl2, 4);
  append (&zahl2, 5);
  append (&zahl2, 8);
  append (&zahl2, 6);
  append (&zahl2, 3);
  append (&zahl2, 0);
  append (&zahl2, 2);
  append (&zahl2, 7);
  append (&zahl2, 3);
  append (&zahl2, 3);
  append (&zahl2, 2);
  append (&zahl2, 2);
  append (&zahl2, 9);
  append (&zahl2, 0);

  //TODO: Addieren Sie die beiden Zahlen mit add(...)

  add (zahl1, zahl2, &result);

  //TODO: Geben Sie das Ergebnis aus

  printList (zahl1);
  printList (zahl2);
  printList (result);

  //TODO: Löschen Sie drei Listen

  delete (zahl1);
  delete (zahl2);
  delete (result);

  //TODO: Lösche Sie die drei Pointer auf die Listen
  free (zahl1);
  free (zahl2);
  free (result);

  return 0;

}
